@@ -24,7 +24,9 @@ int connected=0;
 int timeout=10;
 int want_all_elems=0;
 int max_tabs=32;
+int cont=1;
 struct main_components components;
+extern struct mem_list* mlist;
 
 void disconnect()
 {
@@ -180,6 +182,7 @@ void construct_components()
  components.table_entry = NULL;
  components.data = NULL;
  components.last_selected = NULL;
+ shred_memlist();
 }
 
 
@@ -293,6 +296,7 @@ void refresh_all()
  char winname[STDSTRING];
  strcpy(basen, currentbase);
  disconnect();
+ shred_memlist();
  if (currentbase) free(currentbase);
  currentbase=NULL;
  connection = gtk_connect_db(basen, errstr);
@@ -336,7 +340,7 @@ void destruct_tab_view(GtkWidget* w, gpointer data)
  components.table_view = NULL;
  if (components.rows && components.cols)
     {
-     free_p3c(&components.data, components.rows, components.cols);
+     mfree_p3c(&components.data, components.rows, components.cols);
      components.data = NULL;
      for (i=0;i<rows;i++) for (j=0;j<cols;j++)
        {
@@ -352,6 +356,7 @@ void destruct_tab_view(GtkWidget* w, gpointer data)
  components.cols = -1;
  if (components.scroller_view) gtk_widget_destroy(components.scroller_view);
  components.scroller_view = NULL;
+ shred_memlist();
 }
 
 
@@ -382,7 +387,7 @@ void browse_table(char* tabname)
  int i,j;
  debug("%s:%d",__FILE__,__LINE__);
  if (!connection) error("no_connection %s:%d", __FILE__,__LINE__);
- execute_printf(connection, &ptr, &row, &col, errstr, 1, "SELECT * FROM %s", tabname);
+ mexecute_printf(connection, &ptr, &row, &col, errstr, 1, "SELECT * FROM %s", tabname);
  if (row<0 || col<0)
    {
     gtk_dialog_printf_big("Error executing query:\nReturn from DBMS: %s\n", errstr);
@@ -395,20 +400,23 @@ void browse_table(char* tabname)
  components.cols = col;
  components.rows = row;
  components.data = ptr;
- if (col>64 || row>1024)
+ if (col>32 || row>512)
    {
     gtk_dialog_printf("Result is too big, rows=%d, columns=%d\nwill be truncated!", row, col);
-    if (row>1024) col = (col>16 )?16 :col;
-    else if (row>512)  col = (col>32 )?32 :col;
-    else if (row>256)  col = (col>64 )?64 :col;
-    else             col = (col>128)?128:col;
-    row = (row>1024)?1024:row;
-    col = (col>1024)?1024:col;
+    if (row>512) col = (col>12 )?11 :col;
+    else if (row>256)  col = (col>24 )?24 :col;
+    else if (row>128)  col = (col>48 )?48 :col;
+    else             col = (col>64)?64:col;
+    row = (row>512)?512:row;
+    col = (col>512)?512:col;
    }
  debug("row=%d, col=%d\n",row,col);
  components.table_view = gtk_table_new(row, col, FALSE);
  res_cache.data = components.data;
  for (i=0;i<row;i++)
+ for (j=0;j<col;j++) components.table_entry[i][j] = NULL;
+ create_pbar("Wait while filling table...");
+ for (i=0;i<row;i++)
  for (j=0;j<col;j++)
    {
     components.table_entry[i][j] = gtk_entry_new();
@@ -416,8 +424,11 @@ void browse_table(char* tabname)
     gtk_entry_set_editable(GTK_ENTRY(components.table_entry[i][j]), FALSE);
     gtk_table_attach (GTK_TABLE (components.table_view), components.table_entry[i][j], j, j+1, i, i+1,GTK_FILL,GTK_FILL, 0, 0);
     gtk_widget_show(components.table_entry[i][j]);
+    if (!cont) { gtk_dialog_printf("Interrupted!"); goto out_loop; }
    }
+ out_loop:
  gtk_widget_show (components.table_view);
+ destroy_pbar();
  components.scroller_view = gtk_scrolled_window_new(NULL,NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(components.scroller_view), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(components.scroller_view), components.table_view);
@@ -1224,14 +1235,14 @@ void gtk_table_out(char* title, char*** str_data, int a, int b)
  res_cache.data = str_data;
  res_cache.rows = a;
  res_cache.cols = b;
- if (b>128 || a>1024)
+ if (b>32 || a>512)
    {
     gtk_dialog_printf("Result is too big, rows=%d, columns=%d\nwill be truncated!", a, b);
-    if (a>1024) b = (b>16 )?16 :b;
-    else if (a>512)  b = (b>32 )?32 :b;
-    else if (a>256)  b = (b>64 )?64 :b;
-    else             b = (b>128)?128:b;
-    a = (a>1024)?1024:a;
+    if (a>512) b = (b>12 )?12 :b;
+    else if (a>256)  b = (b>24 )?24 :b;
+    else if (a>128)  b = (b>48 )?48 :b;
+    else             b = (b>64 )?64:b;
+    a = (a>512)?512:a;
    }
  entry = (GtkWidget***)malloc(a*sizeof(GtkWidget**));
  for (i=0;i<a;i++) entry[i] = (GtkWidget**)malloc(b*sizeof(GtkWidget*));
@@ -1513,12 +1524,12 @@ void execute_proc(GtkWidget* w, gpointer txt)
     gtk_dialog_printf_big("Error executing query:\nReturn from DBMS: %s\n", errstr);
     return;
    }
- out = pgres2pc3(result, &a, &b, 1);
+ out = mpgres2pc3(result, &a, &b, 1);
  clear_result(&result);
  if (a>0 && b>0) gtk_table_out("Results of Query:", out, a, b);
  else gtk_dialog_printf("Query was OK, but didn't returned any result\nThis probably was INSERT OR CREATE...");
+ mfree_p3c(&out, a, b);
  refresh_all(); 		/* FIXME HARDCORE !!! */
- free_p3c(&out, a, b);
 }
 
 
@@ -1658,11 +1669,19 @@ void check_exec_proc(GtkWidget* txt, gpointer dlg)
 
 void post_start(GtkWidget* w, gpointer data)
 {
+ debug("%s:%d",__FILE__,__LINE__);
  setenv("PGHOST", "localhost", 1);
  setenv("PGPORT", "5432", 1);
  system("pg_ctl start");
 }
 
+void global_free_mem(GtkWidget* w, gpointer data)
+{
+ debug("%s:%d",__FILE__,__LINE__);
+ destruct_tab_view(NULL,NULL);
+ shred_memlist();
+}
+
 void query_executor(GtkWidget* w, gpointer data)
 {
  GtkWidget* window, *table, *text, *hscrollbar, *vscrollbar, *button[3];
@@ -1734,7 +1753,8 @@ static GtkItemFactoryEntry menu_items[] = {
   { "/File/Quit",     "<control>Q", (void (*)(GtkWidget*,gpointer))menu_quit, 0, NULL }, /* DONE */
   { "/_Options",      NULL,         NULL, 0, "<Branch>" },			/* DONE */
   { "/Options/Test",  "<control>T",         exec_test_prog, 0, NULL },		/* DONE */
-  { "/Options/Start _Postmaster",  "<control>P",post_start, 0, NULL },/* DONE */
+  { "/Options/Start _Postmaster",  "<control>P",post_start, 0, NULL },		/* DONE */
+  { "/Options/Global FreeMem",  NULL,        global_free_mem, 0, NULL },	/* DONE */
   { "/Options/Environment",     NULL,       env_dialog, 0, NULL },		/* DONE */
   { "/_Execute",      NULL,         NULL, 0, "<Branch>" },
   { "/Execute/_SQL Query Editor",  NULL        ,   query_executor, 0, NULL },
@@ -1788,7 +1808,7 @@ int parse_cmdline(int lb, char** par, GtkWidget* w)
  debug("%s:%d",__FILE__,__LINE__);
  gotdb=0;
  dbname = NULL;
- while ((ch = getopt(lb,par,"d:s:p:b:t:hev")) != -1)
+ while ((ch = getopt(lb,par,"d:s:p:b:t:m:hev")) != -1)
    {
     switch (ch)
       {
@@ -1799,6 +1819,7 @@ int parse_cmdline(int lb, char** par, GtkWidget* w)
        case 'd': gotdb=1; dbname = malloc(strlen(optarg)+1); strcpy(dbname, optarg); break;
        case 'e': gotdb=1; env_dialog(w,(gpointer)1); return 0;
        case 'v': want_all_elems=1; break;
+       case 'm': max_tabs = atoi(optarg); if (max_tabs<=0) fatal("max tabs cannot be less or equal 0");  break;
        case 'h':
 		 gtk_dialog_printf_big("Options are:\n-d [database_name]\n"
 				 "-e (use advanced environment)\n"
@@ -1807,10 +1828,11 @@ int parse_cmdline(int lb, char** par, GtkWidget* w)
 				 "-p port\n"
 				 "-b base dir\n"
 				 "-t timeout connection\n"
+				 "-m max_tables in EntireDB\n"
 				 "-v verbose, display all TABLES/VIEWS etc\n"
 				 ); return 1;
        default:
-		gtk_dialog_printf("Unrecognized command line option\noptions are:\n-d data_base_name\n"
+		gtk_dialog_printf("Unrecognized command line option, try:\n"
 				   "-h help"); return 1;
       }
    }
@@ -1848,11 +1870,8 @@ int parse_cmdline(int lb, char** par, GtkWidget* w)
 
 void todo()
 {
- printf("HANDLE ALL BUTTONS IN ENTIREDB CTRL\n");
- printf("SAVE_DB_AS_HTML, SAVE_CURRENT_PAGE_TABLE, (NEXT/PREV)_PAGE\n");
- printf("TABS_POS, SHOW_HEADERS/LABELS, CLOS\n");
- printf("HANDLE CMD LINE FOR max_tabs\n");
- printf("CHECK MEMORY LEAKS!!!\n");
+ printf("TRUNCATED ROWS - CONROL TO SELECT A WINDOW, WITH <--*------>\n");
+ printf("select Number from <a,b> control, z suwakiem!\n");
 }
 
 
