@@ -13,6 +13,7 @@
 #include <libpq-fe.h>
 #include <string.h>
 #include "consoleout.h"
+#include "mmsystem.h"
 PGconn* connect_db(char*);
 PGconn* gtk_connect_db(char*,char*);
 void clear_result(PGresult**);
@@ -21,8 +22,18 @@ PGresult* execute_silent_query(PGconn*,char*);
 PGresult* execute_query(PGconn*,char*);
 void execute_printf_query(PGconn*,PGresult**,char*, char*,...);
 void execute_printf(PGconn*,char****, int*, int*, char*, int, char*,...);
+void mexecute_printf(PGconn*,char****, int*, int*, char*, int, char*,...);
+void mem_execute_printf(struct mem_list**, PGconn*,char****, int*, int*, char*, int, char*,...);
 char*** p3c_execute_printf(PGconn*,int*, int*,char*,int, char*,...);
+char*** mp3c_execute_printf(PGconn*,int*, int*,char*,int, char*,...);
+char*** mem_p3c_execute_printf(struct mem_list**, PGconn*,int*, int*,char*,int, char*,...);
 char*** pgres2pc3(const PGresult*,int*,int*, int);
+char*** mpgres2pc3(const PGresult*,int*,int*, int);
+char*** mem_pgres2pc3(struct mem_list**,const PGresult*,int*,int*, int);
 void free_p3c(char****,int,int);
+void mfree_p3c(char****,int,int);
+void mem_free_p3c(struct mem_list**,char****,int,int);
 int get_db_tables(PGconn*,char***, int*,int);
+int mget_db_tables(PGconn*,char***, int*,int);
+int mem_get_db_tables(struct mem_list**, PGconn*,char***, int*,int);
 #endif
