@@ -9,6 +9,7 @@
 #include "common.h"
 #include "signals.h"
 extern int ping_conn;
+extern int cont;
 
 void priv_sig_proc()
 {
@@ -20,6 +21,13 @@ void priv_sig_proc()
 }
 
 
+void sig_abort()
+{
+ debug("%s:%d", __FILE__,__LINE__);
+ cont=0;
+}
+
+
 void alarm_clock(int sec)
 {
  debug("%s:%d", __FILE__,__LINE__);
@@ -37,11 +45,14 @@ void alarm_stop()
 void setup_signals()
 {
  static struct sigaction act;
+ static struct sigaction acu;
  ping_conn=0;
  debug("%s:%d", __FILE__,__LINE__);
  act.sa_handler = priv_sig_proc;
  sigfillset(&(act.sa_mask));
- sigfillset(&(act.sa_mask));
  sigaction(SIGALRM, &act, NULL);
+ acu.sa_handler = sig_abort;
+ sigfillset(&(acu.sa_mask));
+ sigaction(SIGUSR1, &acu, NULL);
 }
 
